/****************************************************************/
/*  
     ***   ***  ***  *** ***     *** ***   ******  ****  ***
     ***   ***  ***  *** ***     *** **   ***  *** ***** ***
     ***   ***  ***  *** ***     *****    ******** *** *****
      *** ***   ***  *** ******* *** **   ***  *** ***  ****
       *****     *****   ******* *** ***  ***  *** ***   ***
	   
	 Vulkan Graphics API: 11 Steps

     Minimalistic examples to build upon and experiment with 
	 to develop an understanding of the underlying API

	   
/*
/****************************************************************/

// Vertex Shader

// shader version
#version 420

// input matrix transforms (3D) passed in from our program
layout ( std140, set = 0, binding = 0 ) uniform buffer 
{
  mat4 inProjectionMatrix;
  mat4 inViewMatrix;
  mat4 inModelMatrix;
};


// input from your program
layout( location = 0 ) in vec4 inPos;
layout( location = 1 ) in vec3 inColor;
layout( location = 2 ) in vec2 inTexCoords;
layout( location = 3 ) in vec3 inNormal;

// output to next stage (fragment)
layout( location = 0 ) out vec4 outColor;
layout( location = 1 ) out vec2 outTexCoords;
layout( location = 2 ) out vec3 outNormal;
layout( location = 3 ) out vec3 outPos;

// shader entry point
void main() 
{	
  // combine the matrices
  mat4 mvp = inModelMatrix * inViewMatrix * inProjectionMatrix;

  // transform position by matrices
  gl_Position = vec4(inPos.xyz, 1) * mvp;

  // output position in view space (or eye space)
  outPos = vec3(vec4(inPos.xyz, 1) * inModelMatrix * inViewMatrix);

  // pass input colour to the next stage
  outColor = vec4(inColor, 1.0);

  // pass input texture coordinates onto the fragment shader
  outTexCoords = inTexCoords;

  // pass input normals onto the fragment shader
  outNormal = mat3(transpose(inverse(inModelMatrix))) * inNormal; 

}// End main(..)